#-*- coding: utf-8 -*-

#***********************************************
# R2rpm
#
# Class file for the class Build
#
#
# Made the 07th March 2010
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#***********************************************

import subprocess, os, sys
from R2rpmError import *
from Packager import *

class Build:
    """ Build class
            Builds the specfile and handles the errors
    """
    
    def __init__(self):
        self.specname = ''
        self.outcode = None
        self.rpm = None
        self.log = None

    def build(self, specname, rpmarg = 'ba', mock = False):
        ''' Builds the given specfile and returns the errors if any '''
        print "* Building the rpm (this may take some time)"
        if rpmarg is None: rpmarg = 'ba'
        if mock is False:
            cmd = "   LANG=C rpmbuild -%s %s > build-%s.log 2>&1" %(rpmarg, 
                    specname, specname)
            self.log = "build-%s.log" %specname
            print cmd
            self.outcode = subprocess.call(cmd , shell=True)
        else:
            if '.cfg' in mock: mock - mock.replace('.cfg','')
            cmd = "   LANG=C rpmbuild -bs %s > build-%s.log 2>&1" %(specname, specname)
            self.log = "build-%s.log" %specname
            print cmd
            self.outcode = subprocess.call(cmd , shell=True)
            self.getRpm(specname)
            packager = Packager()
            packager.readConfFile()
            mockcommand = packager.mock %(mock, self.rpm[0])
            cmd = "   LANG=C %s >> build-%s.log 2>&1" %(mockcommand, specname)
            self.log = "/var/lib/mock/%s/result/build.log" %mock
            if os.path.exists(self.log):
                os.remove(self.log)
            print cmd
            self.outcode = subprocess.call(cmd , shell=True)
        #print self.outcode
    
    def getRpm(self, specname):
        ''' Parses the build.log to extract the rpm generated '''
        f = open(self.log, 'r')
        log = f.read()
        f.close
        logs = log.split("\n")
        for line in logs:
            if 'Wrote: ' in line and '.rpm' in line:
                rpm = line.split('Wrote:')[1].strip()
                if self.rpm is None:
                    self.rpm = []
                    self.rpm.append(rpm)
                else:
                    self.rpm.append(rpm)
        
    
    def parseError(self, specname):
        ''' Parses the build.log to extract the error 
                Generates a file list (file installed but not found)
                Throw an error Package not found
                Throw an error Build Time error
        '''
        try:
            f = open(self.log, 'r')
            log = f.read()
            f.close
        except IOError, err:
            print '* ERROR: Something wrong happened during the building process'
            print err
            print "Quitting"
            sys.exit(1)
            
        logs = log.split("\n")
        
        flagfile = False
        flagdep = False
        self.files = [] # stores the files for the %file section
        self.deps = [] # stores missing dependencies
        for row in logs:
            if row == "":
                flagfile = False
            if 'Child returncode was: 1' in row or 'Installed' in row or 'Failed' in row:
                flagfile = False
            if flagfile:
                self.files.append(row)
            if flagdep:
                entry = row.split('is needed')[0].replace('\t','').strip()
                if entry not in self.deps:
                    self.deps.append( entry )
            if 'Installed (but unpackaged) file(s) found:' in row:
                flagfile = True
            if 'Failed build dependencies:' in row:
                flagdep = True
        
        if len(self.deps) != 0:
            dep = ", ".join(self.deps)
            raise BuildDepencenciesError("Dependencies missing: %s"%dep)
        
        if len(self.deps) == 0 and len(self.files) == 0 and self.outcode == 1:
            log = os.getcwd() + "/build-%s.log" %specname
            raise BuildError("An error occured at build time, see the log in %s"%log)
        
        return self.files
        
