#-*- coding: utf-8 -*-

#***********************************************
# R2rpm
#
# R2rpm generates RPM using the R2spec library
# building the RPM and adjusting the spec accordingly
#
#
# Made the 07th March 2010
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#***********************************************

import getopt, sys, re, os
from r2spec.RPackage import RPackage
from r2spec.Build import Build
from r2spec.Spec import Spec

USAGE ="""
This program is designed to create RPM from a source file or a url using R2spec.
Usage:  R2rpm -u <url> 
        R2rpm -s <source>
        R2rpm -s folder/folder/<source>
        R2rpm -p <name>
        R2rpm -p <name> --mock=fedora-12-i386

You can also use the following options to add the correct Source0 in the spec file:
  --bioc                For Bioconductor libraries
  --cran                For libraries coming from the CRAN repository
  --rforge              For libraries coming from the r-forge.net
  --rproject            For libraries coming from the r-forge.r-project.org

The source can be in the same directory or in a subdirectory.

The information about the packager:
- Fill /etc/R2spec.conf 
- Copy and fill /etc/R2spec to ~/.R2spec.conf
- Use the option -n/--name and -e/--email to set the corresponding values
"""

#################################################
### main
# Do the job :)
class R2rpm:
    """ R2rpm class
            Enables to use this code from another script
    """
    
    def __init__(self):
        ''' Instanciate the default values for the attributes '''
        self.built = []
    
    def main(self, source, url, package, bioc, cran, rforge, rproject, \
            copyFile, dependencies, name, email, force, mock):
        ''' Performs the work'''
        
        from r2spec.R2spec import R2spec
        
        # Generate the spec file
        r = RPackage()
        # Determine where to generate the spec according to the rpmmacro
        specdir = r.getRPMTopDirectory("_specdir", package)
        
        # Go to the folder
        if specdir is not None:
            os.chdir(specdir)
        else:
            print "Cannot change to the spec folder, using the current folder instead."
        # Enforce the copyfile and the force option
        r = R2spec()
        r.todo = False
        r.usage = USAGE
        specname = r.main(source, url, package, bioc, cran, rforge, rproject, \
                    True, name, email, True)
        specname = specname + ".spec"
        self.built.append(specname)
        
        print "*"*50
        print "** Spec generated, starting RPM construction"
        # Remove the content of the %file section
        spec = Spec()
        spec.readSpec(specname)
        spec.cleanFiles()
        spec.writeSpecTosystem(specname)
        # Generate the rpm from the cleaned spec file
        rpmarg = r.package.packager.rpmarg
        self.generateRpm(spec, specname, dependencies, rpmarg, mock)
        
        
    def generateRpm(self, spec, specname, dependencies = False, rpmarg ='ba', mock = False):
        ''' Generate the rpm '''
        from r2spec.R2rpmError import R2rpmError, BuildError, BuildDepencenciesError
        
        # Build
        b = Build()
        try:
            # First build
            b.build(specname, rpmarg, mock)
            # Parse the output
            files = b.parseError(specname)
            if len(files) != 0:
                # Add the file to the file section
                spec.addFile(files)
                # Write down the spec file
                spec.writeSpecTosystem(specname)
            # Rebuild the package when build has failed the first time
            if b.outcode: # == 1 when build failed and 0 when build passed
                self.generateRpm(spec, specname, mock)
        except BuildDepencenciesError, err:
            print "An error occured during the build"
            print err
            if dependencies:
                print "\n* Building dependencies"
                for package in b.deps:
                    print package
            sys.exit(1)
        except BuildError, err:
            print "An error occured during the build"
            print err
            sys.exit(1)
        
        if b.outcode == 0 :
            print "** RPM %s done **" %specname.split('.spec')[0]
            # Get the list of rpm generated
            b.getRpm(specname)
            print "\n".join(b.rpm)
