#-*- coding: utf-8 -*-

#***********************************************
# R2spec 
#
# R2spec is made to help to the creation of R specfile
# It works from a source file (*.tar.gz) or a url (http://...*.tar.gz)
#
#
# Made the 13th February 2008
# by Pierre-Yves chibon
#
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
# This software has been based on the guidelines for R packaging
# http://fedoraproject.org/wiki/Packaging/R
#
#***********************************************

import sys, re, os
from r2spec.RPackage import RPackage

USAGE ="""
This program is designed to create specfile from a source file or a url.
Usage:  R2spec -u <url> 
        R2spec -s <source>
        R2spec -s folder/folder/<source>
        R2spec -p <name>

You can also use the following options to add the correct Source0 in the spec file:
  --bioc                For Bioconductor libraries
  --cran                For libraries coming from the CRAN repository
  --rforge              For libraries coming from the r-forge.net
  --rproject            For libraries coming from the r-forge.r-project.org

The source can be in the same directory or in a subdirectory.

The information about the packager:
- Fill /etc/R2spec.conf 
- Copy and fill /etc/R2spec to ~/.R2spec.conf
- Use the option -n/--name and -e/--email to set the corresponding values
"""

#################################################
### main
# Do the job :)
class R2spec:
    """ R2spec class
            Enables to use this code from another script
    """
    
    def __init__(self):
        self.usage = USAGE
        self.todo = True
    
    def main(self, source, url, package, bioc, cran, rforge, rproject, copyFile, name, email, force):
        ''' Performs the work'''
        
        # Return the usage message if no arguments are given
        if(url == None and source == None and package == None):
            sys.exit(self.usage)

        # Return an error message if there is 2 arguments given
        if ([url, source, package].count(None) < 2):
            sys.exit("\n***  Please give only one argument  ***\n")

        # Fails if a library comes from both bioconductor and cran
        if ([cran, bioc, rforge, rproject].count(True) > 1):
            sys.exit("\n***  You cannot set several origin at the same time (cran/bioc/r-forge/r-forge.r-project)  ***\n")
        
        # For a given package name, set the url by looking at the repository
        if (package != None):
            p = RPackage()
            (specurl, url) = p.findSourceOfPackage(package)
            if specurl is None:
                sys.exit("Could not find the package: %s"%package)

        # Check wether the url or the source exist and if so check wether they are http:// and tar.gz
        if(url != None):
            # Check wether the url start with http://
            if not url.startswith('http://'):
                sys.exit("\nYour URL does not start with 'http://' => %s \n" % url)

            # Check wether the url refers to a .tar.gz
            if not url.startswith('http://') and not url.endswith('.tar.gz'):
                sys.exit("Your URL does not link to a tarball file %s \n" % url)

        if(source != None):
            if not source.endswith('.tar.gz') and not source.endswith('.zip'):    
                sys.exit("Your source is not a tarball (*.tar.gz) or a zip file (*.zip) %s \n" % source )
            if not os.path.exists(source):
                sys.exit("Your source does not exist, please use the full path (%s)"%source)

        package = RPackage()
        # Set the url or source (tarball)
        package.setSource(url, source)
        # Retrieve the package
        package.download()
        # Get the information regarding the packager
        package.setPackager(bioc, name, email)
        # Set the name of the package
        package.setName()
        # If it is a package from one of the standard repository, set the URL
        if url is not None: package.setUrl(url)
        elif bioc: package.setBiocUrl()
        elif cran: package.setCranUrl()
        elif rforge: package.setRForgeURL()
        elif rproject: package.setRForgeRProjectURL()
        # Open the package into a tmp directory
        package.openSource()
        # Set the meta information of the spec (license, version, summary, description...)
        package.setDescription()
        # Set the arch (noarch vs *86)
        package.getArch()
        # Remove the tmp directory in which the sources have been extracted
        package.closeSource()
        # Set the date to be used at the bottom of the changelog
        package.setDate()
        print "*"*50,"\n** Generate the spec file"
        package.writeSpec(force, self.todo)
        package.moveSource(copyFile)
        self.package = package
        
        return "R-" + package.name

#if __name__ == "__main__":
    # Retrieve the given arguments
    #R2spec().main(None, None, 'ABarray')
