#-*- coding: utf-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Package
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

from Packager import *
from Description import *
from Spec import *
from subprocess import Popen, PIPE

import os, sys, re, datetime

class RPackage:
    '''
    Package class.
    It contains all the information about the package that R2spec will create the spec file about.
    '''

    def __init__(self):
        ''' Initiate the object of type Package and set the attributes '''
        self.name = None
        self.source0 = None
        self.URL = None
        self.version = None
        self.date = None
        self.packager = None
        self.description = None
        self.source = None
        self.path = None
        self.created = None
        self.arch = None
        self.topdir = None
        self.sourcedir = None
        self.specdir = None
    
    def setName(self):
        ''' Get the name of the library from the source file '''
        name = self.source
        name = name.split('_')[0]
        try:
            self.name=name.rsplit('/',1)[1]
        except IndexError, err:
            self.name=name
        
        print "* Name of the library =", self.name

    def setDate(self):
        ''' Get the current date with the correct format for the spec file '''
        date= datetime.datetime.now().strftime("%a %b %d %Y")
        self.date = date
        print "* Date of the spec file = %s" %self.date
   
    def setSource(self, source0, source):
        ''' From the url retrieve the name of the source '''
        if source != None and source0 != None:
            if source != source0.rsplit('/',1)[1] :
                sys.exit("Do not give 2 contradictory sources of information")
        elif source != None:
            try:
                self.source = source.rsplit('/',1)[1]
                self.path = source.rsplit('/',1)[0]
            except IndexError, err:
                self.source = source
                self.path = '../'
        elif source0 != None:
            self.source = source0.rsplit('/',1)[1]
            self.path = '../'
            self.source0 = source0
        else:
            sys.exit("The source are not correct ")
    
    def download(self):
        ''' If the package has an url download the source file '''
        print "\n* Retrieve the source"
        if self.source0 != None:
            source=self.source0.rsplit('/',1)[1]
            # Check the presence of the source, download only if they are not already in the working directory
            exist = os.path.isfile(source)
            if exist == True:
                print "The file %s already exist, there is no need to download it again." %(self.source)
            else:
                cmd = "wget "+self.source0
                print cmd
                os.system(cmd)

    def openSource(self):
        '''
        Create a tmp directory where the tarball is opened and open
        the tarball.
        '''
        # Creates the tmp directory in which the sources are opened
        self.created = 1
        try:
            os.mkdir('tmp')
        except Exception, err:
            self.created = 0
        #print "The folder tmp already exist, no need to create it"

        # Go to the tmp directory
        os.chdir('tmp/')
        
        # Go into the tmp folder and untar the source
        if self.source.endswith('tar.gz'):
            cmd = "tar -xvf " + self.path + "/" + self.source + " >/dev/null"
        elif self.source.endswith('zip'):
            cmd = "unzip %s/%s >/dev/null" %(self.path, self.source)
        else:
            sys.exit("Your source is not a supported tarball (*.tar.gz or *.zip) %s \n" % source )
 
        #print cmd
        (os.system(cmd)) #or (sys.exit("The source can not be found "+source))
        os.chdir('..')

    def closeSource(self):
        '''
        Remove the tmp directory and the folder of the tarball if the 
        tmp directory has been created else just remove the folder of
        the tarball
        '''
        #print os.getcwd()
        cmd = "./tmp/"+self.name
        shutil.rmtree(cmd)
        if self.created == 1:
            try:
                os.rmdir('tmp')
            except IOError, err:
                sys.exit('tmp directory can not be removed')
    
    def setDescription(self):
        ''' Create the object description and fill the attributes '''
        description = Description()
        description.setDescriptionFile(self)
        
        description.setVersion()
        print "* Version of the library = %s" %description.version
        description.setSummary()
        print "* Summary of the library = %s" %description.summary
        description.setDescription()
        print "* Description of the library =\n %s " %description.description
        description.setLicense()
        print "* License of the library = %s" %description.license
        description.setRequires()
        
        self.description = description
    
    def getArch(self):
        ''' Determine is the package is of the type Noarch or Arch '''
        # Save the top directory
        original_directory = os.getcwd()

        # Change directory and look for files
        os.chdir("./tmp/"+self.name)
        # Look for the files and put them into a tmp file
        cmd = 'find ./ -name "*.c" -o -name "*.C" -o -name "*.cp" -o -name "*.cpp" -o -name "*.f" -o -name "*.F" -o -name "*.h" -o -name "*.H" > tmp'
        os.system(cmd)
        # Get the size of the generated file
        size = os.path.getsize('tmp')
        # If the size of this file is 0 the package should probably be noarch
        if size == 0:
            self.arch = 'noarch'
            print "* The package is = Noarch" 
        else:
            print "* The package is = Arch ~ x86 or x86_64" 
        # Remove this file
        os.system('rm tmp')
        
        # Comes back to the original directory
        os.chdir(original_directory)
        
    def setPackager(self, bioc, name, email):
        ''' Creates the object packager and fill the attributes '''
        packager = Packager()
        packager.readConfFile(bioc)
        #print name, email
        if name:
          packager.name = name
        if email:
          packager.email = email

        self.packager = packager
        print "\n","*"*50
        print "* Name of the packager =", self.packager.name
        print "* Mail of the packager =", self.packager.email    
        print "\n","*"*50

    def setUrl(self, url):
        ''' Set the given url in the URL fiel of the spec file '''
        self.URL = url

    def setBiocUrl(self):
        ''' Set the url of a standard Bioconductor package if the option has been used '''
        if self.source0 is None:
            self.source0 = "http://bioconductor.org/packages/release/bioc/src/contrib/%{packname}_%{version}.tar.gz"
        self.URL = "http://bioconductor.org/packages/release/bioc/html/" + self.name + ".html"


    def setCranUrl(self):
        ''' Set the url of a standard Cran package if the option has been used '''
        if self.source0 is None:
            if self.source != None:
                urlCran = "http://cran.r-project.org/src/contrib/"+self.source
            else:
                urlCran = "http://cran.r-project.org/src/contrib/%{packname}_%{version}.tar.gz"
            self.source0 = urlCran
        self.URL = "http://cran.r-project.org/web/packages/" + self.name + "/index.html"

    def setRForgeURL(self):
        ''' Set the url of a standard RForge.net package if the option has been used '''
        if self.source0 is None:
            if self.source != None:
                urlRForge = "http://www.rforge.net/src/contrib/%s" %self.source
            else:
                urlRForge = "http://www.rforge.net/src/contrib/%{packname}_%{version}.tar.gz"
            self.source0 = urlRForge
        self.URL = "http://www.rforge.net/%s/" %self.name
    
    def setRForgeRProjectURL(self):
        ''' Set the url of a standard r-forge.r-project.org package if the option has been used '''
        if self.source0 is None:
            if self.source != None:
                urlRForgeRProject = "http://r-forge.r-project.org/src/contrib/%s" %self.source
            else:
                urlRForgeRProject = "http://r-forge.r-project.org/src/contrib/%{packname}_%{version}.tar.gz"
            self.source0 = urlRForgeRProject
        self.URL = "http://r-forge.r-project.org/projects/%s/" %self.name

    def writeSpec(self, force, todo = True):
        ''' Check if the spec file already exist and generate it '''
    
        # Do something if the spec file already exist
        specname = "R-" + self.name + ".spec"
        if specname in os.listdir('./') and not force:
            text =  "The specfile R-" + self.name + ".spec already exist !\nDo you want to overwrite it ? Y/N [N]\n"
            answer = raw_input(text)
            if answer == '':
                answer = "N"
            if answer.lower() == "n" or answer.lower() == "no":
                if specname+"-new" not in os.listdir('./'):
                    specname = specname+"-new"
                elif specname+"-new1" not in os.listdir('./'):
                    specname = specname+"-new1"
                else:
                    text = "There are already several version of the specfile in this directory, please indiquate the name you would like to give:\n"
                    specname = raw_input(text)
        
        spec = Spec(self)
        spec.finishName(self.arch)
        spec.writeSpec(specname)
        if todo: spec.writeOut()
    
    def getRPMTopDirectory(self, tag, package = None):
        ''' Reads the .rpmmacros and set the values accordingly
            Code from José Matos.
        '''
        dirname = Popen(["rpm", "-E", '%' + tag], stdout=PIPE).stdout.read()[:-1]
        # replace %{name} by the specname since it is not set

        # determine the spec name
        if self.name: specname = "R-" + self.name
        elif package: specname = "R-" + package
        else:         specname = "R-unknown"

        # replace %{name} by the specname
        dirname = dirname.replace("%{name}", specname)
        dirname = dirname.replace("%name", specname)

        return dirname
        
    def moveSource(self, copyFile):
        ''' Moves the tarball to where it should be to build the RPM '''
        sourcedir = self.getRPMTopDirectory("_sourcedir")
        
        if sourcedir is not None:
            if copyFile == False:
                text = "Do you want to copy the source (" + self.source + ") to " + sourcedir + "? Y/N [Y]\n"
                answer = raw_input(text)
            else: answer = "y"
            if answer == '':
                answer = "y"
            if answer.lower() == "y" or answer.lower() == "yes":
                print "cp ./" + self.source, "to", sourcedir
                try:
                    shutil.copy(self.source, sourcedir)
                    print "Done"
                except Exception, err:
                    print "Can not copy the source to ",sourcedir
                    print err
            else:
                print "Done"
    
    def findSourceOfPackage(self, name):
        ''' For a given name find in which repository the source are and returns the corresponding url.
            It uses the PACKAGES file of the repository for this.
        '''
        print "* Search source"
        import urllib, re
        dicrepo = {
                'http://r-forge.r-project.org/projects/%s/':'http://r-forge.r-project.org/src/contrib/PACKAGES',
                'http://cran.at.r-project.org/web/packages/%s/index.html':   'http://cran.at.r-project.org/src/contrib/PACKAGES',
                'http://www.bioconductor.org/packages/release/bioc/html/%s.html':     'http://www.bioconductor.org/packages/release/bioc/src/contrib/PACKAGES',
                'http://www.bioconductor.org/packages/release/data/experiment/html/%s.html':     'http://www.bioconductor.org/packages/release/data/experiment/src/contrib/PACKAGES',
                'http://www.bioconductor.org/packages/release/data/annotation/html/%s.html':     'http://www.bioconductor.org/packages/release/data/annotation/src/contrib/PACKAGES',
        }
        
        for repo in dicrepo.keys():
            f = urllib.urlopen(dicrepo[repo])
            s = f.read()
            f.close()
            sourcemotif = re.compile("Package:\s+%s\n"%name)
            result = sourcemotif.search(s)
            if result is not None:
                print "Ok %s" %(repo %name)
                versionmotif = re.compile("Package:\s+%s\nVersion:(.*)"%name)
                version = versionmotif.search(s).group(1).strip()
                url = dicrepo[repo].split('PACKAGES')[0] + "%s_%s.tar.gz" %(name, version)
                return (repo %name, url)
            else:
                print "Not Found: %s" %(repo %name)
        
        return (None, None)
