#-*- coding: utf-8 -*-

#**********************************************
# R2spec 
#
# Class file for the class Spec
#
# Distributed under License GPLv3 or later
# You can find a copy of this license on the website
# http://www.gnu.org/licenses/gpl.html
#
#**********************************************

import os, sys
from jinja2 import Template

class Spec:
    """
    Spec Class
        Write the spec file.
    """
    
    def __init__(self, package = None):
        self.package = package
        self.dict = {}
        self.spec = ""    
        self.name = ""
    
    def finishName(self, arch):
        ''' Generate the dictionnary use for the template
            and complete the information while not found '''
        
        if self.package.packager.name is None or self.package.packager.name == "":
            self.dict['name'] = 'Name'
        else:
            self.dict['name'] = self.package.packager.name
        
        if self.package.packager.email is None or self.package.packager.email == "":
            self.dict['email'] = 'Mail'
        else:
            self.dict['email'] = self.package.packager.email
        
        if self.package.source0 is None:
            self.package.source0 = ''
        else:
            self.dict['source0'] = self.package.source0
        
        if self.package.URL is None:
            self.package.URL = ''
        else:
            self.dict['URL'] = self.package.URL
            
        if self.package.description.summary is None or self.package.description.summary == '':
            print " /!\ Summary empty /!\ "
            self.dict['summary'] = "<      summary should be not be longer than this       >"
        else:
            self.dict['summary'] = self.package.description.summary
        
        if self.package.description.description is None or self.package.description.description =='':
            print " /!\ Description empty /!\ "
            self.dict['description'] = ''
        else:
            self.dict['description'] = self.package.description.description
        
        if arch == None or arch == '':
            self.dict['arch'] = 'arch'
        else:
            self.dict['arch'] = arch
        
        # other options
        self.dict['version'] = self.package.description.version
        self.dict['packname'] = self.package.name
        self.dict['license'] = self.package.description.license 
        self.dict['date'] = self.package.date 
        self.dict['depends'] = self.package.description.depends
        self.dict['suggests'] = self.package.description.suggests
    
    def writeSpec(self, specname):
        ''' Write the spec file '''
                
        print "* Spec generated: \n  %s/%s" %(os.getcwd(), specname)
        try:
            tplfolder = os.path.dirname(__file__)
            f = open(tplfolder + '/specfile.tpl', 'r')
            tplfile = f.read()
            f.close()
            mytemplate = Template(tplfile)
            self.spec = mytemplate.render(self.dict)
        except IOError, err:
            print 'Cannot read the file %s' %tplfolder + '/specfile.tpl'
            print err
            sys.exit(1)

        self.writeSpecTosystem(specname)


    def writeOut(self):
        ''' Print final advices '''
        print "*"*50
        print """TODO:
 * Check Summary
 * Check description
 * Check license
 * Add/Check Source0 and URL
 * Check Arch - We detected it as Arch package but it needs to be checked, use rpmlint for it
 * Change Requires and BuildRequires
 * Change %files
 """

    def readSpec(self, specname):
        ''' Read a given spec and keep it in memory '''
        print '* Read the file %s' %specname
        self.name = specname
        try:
            f = open(self.name , 'r')
            self.spec = f.read()
            f.close()
        except IOError, err:
            print 'Cannot read the file %s' %self.name
            print err
        
    
    def cleanFiles(self):
        '''Remove all the content of the %files section'''
        print "* Clean the %file section"
        cnt = 0
        remove = False
        self.spec = self.spec.split("\n")
        while cnt < len(self.spec):
            row = self.spec[cnt]
            if row.startswith('%changelog'):
                remove = False
            if remove:
                del self.spec[cnt]
                cnt = cnt -1
            if row.startswith('%defattr'):
                remove = True
            cnt = cnt + 1
        
        self.spec = "\n".join(self.spec)
    
    def writeSpecTosystem(self, specname):
        ''' Write the spec in memory with the given name'''
        try:
            specfile = open(specname, 'w')
            specfile.write(self.spec)
            specfile.close()
            print "* spec file written"
        except IOError, err:
            print 'Cannot write the file %s' %specname
            print err    

    def addFile(self, files):
        ''' Add to a spec file the given files list '''
        print "* Add files to the %file section"
        
        files.reverse()
        files = self.generateFiles(files)
        
        cnt = 0
        add = False
        self.spec = self.spec.split("\n")
        while cnt < len(self.spec):
            row = self.spec[cnt]
            if row.startswith('%changelog'):
                self.spec.insert(cnt, "")
                cnt = cnt + 1
                add = False
            if row.startswith('%defattr'):
                add = True
            if add:
                for file in files:
                    cnt = cnt + 1
                    self.spec.insert(cnt, file)
            cnt = cnt + 1
        
        self.spec = "\n".join(self.spec)

    def generateFiles(self, files):
        ''' Generate the list of files to add to the spec '''
        
        macros = {
                '/usr/share/': '%{_datadir}/',
                '/usr/lib/': '%{_libdir}/',
                '/usr/lib64/': '%{_libdir}/',
                }

        doc = ['doc','html','description','news','copying','licence','citation']
        
        packname = self.name.split('.spec')[0].split('R-')[1]
        
        linedocs = []
        linefiles = []
        for filename in files:
            folder, name = filename.split('R/library/%s/' %packname)
            name = name.strip().split('/')[0]
            folder = folder.strip()
            for macro in macros.keys():
                folder = folder.replace(macro, macros[macro])
            if name.lower() in doc:
                linedocs.append("%doc " + folder + "R/library/%s/"%packname  + name)
            else:
                linefiles.append(folder + "R/library/%s/"%packname  + name)
        
        fileout = []
        for doc in set(linedocs):
            fileout.append(doc)
        for file in set(linefiles):
            fileout.append(file)
        
        return fileout
                
